import numpy as np
import xgboost as xgb
import argparse
from sklearn import datasets
import os
from joblib import Memory
from time import time

rng = np.random.RandomState(1994)

PARAMETERS = {'nthread': os.cpu_count()}
memory = Memory('./cachedir', verbose=0)


@memory.cache
def make_regression(args):
    X, y = datasets.make_regression(n_samples=args.samples,
                                    n_features=args.features,
                                    random_state=rng)
    return X, y


@memory.cache
def make_classification(args):
    X, y = datasets.make_classification(n_samples=args.samples,
                                        n_features=args.features,
                                        n_classes=args.classes,
                                        n_informative=args.classes,
                                        random_state=rng)
    return X, y


def configure(args, parameters):
    tm = args.tree_method
    if tm == 'approx':
        parameters.update({'tree_method': 'approx'})
    elif tm == 'local':
        parameters.update({'updater': 'grow_local_histmaker,prune'})
    elif tm == 'sketch':
        parameters.update({'updater': 'grow_skmaker,prune'})
    elif tm == 'hist':
        parameters.update({'tree_method': 'hist'})
    print('Parameters:', parameters)
    return parameters


def regression(args):
    X, y = make_regression(args)
    m = xgb.DMatrix(X, y)
    evals_results = {}
    parameters = configure(args, PARAMETERS.copy())
    booster = xgb.train(parameters,
                        dtrain=m,
                        evals=[(m, 'Train')],
                        evals_result=evals_results)
    print(evals_results)


def classification(args):
    X, y = make_classification(args)
    m = xgb.DMatrix(X, y)
    evals_results = {}
    parameters = configure(args, PARAMETERS.copy())
    parameters.update({'num_class': args.classes})
    booster = xgb.train(parameters,
                        dtrain=m,
                        evals=[(m, 'Train')],
                        evals_result=evals_results)
    print(evals_results)


def main(args):
    start = time()
    if args.task == 'reg':
        regression(args)
    elif args.task == 'cls':
        classification(args)
    else:
        raise ValueError('Unknown task:', args.task)
    end = time()
    print('Duration:', end - start)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--samples', type=int, default=int(1e5))
    parser.add_argument('--features', type=int, default=10)
    parser.add_argument('--classes', type=int, default=4)
    parser.add_argument('--task',
                        type=str,
                        default='reg',
                        choices=['reg', 'cls'])
    parser.add_argument('--tree_method',
                        type=str,
                        choices=['sketch', 'local', 'hist', 'approx'],
                        required=True)
    args = parser.parse_args()
    main(args)
